@@ -36,6 +36,7 @@
 #include "dbus-private.h"
 #include "gvariant-private.h"
 #include "private.h"
+#include "useful.h"
 
 #define NODE_TYPE_CALLBACK	L_DBUS_MATCH_NONE
 
@@ -240,12 +241,12 @@ unsigned int _dbus_filter_add_rule(struct _dbus_filter *filter,
 				void *user_data)
 {
 	struct filter_node **node_ptr = &filter->root;
-	struct filter_node *uninitialized_var(node);
+	struct filter_node *node;
 	struct filter_node *parent = filter->root;
 	bool remote_rule = false;
 	struct _dbus_filter_condition sorted[rule_len];
 	struct _dbus_filter_condition *unused;
-	struct _dbus_filter_condition *uninitialized_var(condition);
+	struct _dbus_filter_condition *condition;
 	struct _dbus_filter_condition *end = sorted + rule_len;
 
 	memcpy(sorted, rule, sizeof(sorted));
@@ -308,6 +309,8 @@ unsigned int _dbus_filter_add_rule(struct _dbus_filter *filter,
 
 		}
 
+_Pragma("GCC diagnostic push")
+_Pragma("GCC diagnostic ignored \"-Wmaybe-uninitialized\"")
 		/*
 		 * Mark the condition used.  We do this by setting
 		 * condition->type to an invalid value unless it is the
@@ -329,6 +332,7 @@ unsigned int _dbus_filter_add_rule(struct _dbus_filter *filter,
 		 * have yet created an AddMatch rule on the server.
 		 */
 		remote_rule |= node->match.remote_rule;
+_Pragma("GCC diagnostic pop")
 	}
 
 	node = l_new(struct filter_node, 1);
@@ -345,8 +349,11 @@ unsigned int _dbus_filter_add_rule(struct _dbus_filter *filter,
 						rule, rule_len))
 			goto err;
 
+_Pragma("GCC diagnostic push")
+_Pragma("GCC diagnostic ignored \"-Wmaybe-uninitialized\"")
 		parent->id = node->id;
 		parent->match.remote_rule = true;
+_Pragma("GCC diagnostic pop")
 	}
 
 	return node->id;
